@@ -55,15 +55,17 @@ class RarArchiver:
     def __init__(self, path: Path) -> None:
         self.path = path
 
-    def read_file(self, archive_file: str) -> bytes:
+    def read_file(self, archive_file: str) -> Optional[bytes]:
         """Read the contents of a comic archive"""
         try:
-            data = bytes()
             with rarfile.RarFile(self.path) as rf:
-                data = rf.read(archive_file)
+                data: bytes = rf.read(archive_file)
             return data
         except rarfile.RarCannotExec as e:
             raise RarError(e) from e
+        except io.UnsupportedOperation:
+            """If rar directory doesn't contain any data, return None."""
+            return None
 
     def remove_file(self) -> bool:
         """Rar files are read-only, so we return False."""
@@ -369,10 +371,7 @@ class ComicArchive:
         page_list = self.get_page_name_list()
 
         num_pages = len(page_list)
-        if num_pages == 0 or index >= num_pages:
-            return None
-
-        return page_list[index]
+        return None if num_pages == 0 or index >= num_pages else page_list[index]
 
     def is_image(self, name_path: Path) -> bool:
         suffix_list = [".jpg", ".jpeg", ".png", ".gif", ".webp"]
