@@ -47,24 +47,28 @@ class ImageMetadata(TypedDict, total=False):
 
 
 @dataclass
-class CreditMetadata:
-    person: str
-    role: str
-    primary: bool
+class GeneralResource:
+    name: str
+    id_: Optional[int] = None
 
 
 @dataclass
-class SeriesMetadata:
-    name: str
+class RoleMetadata(GeneralResource):
+    primary: bool = False
+
+
+@dataclass
+class SeriesMetadata(GeneralResource):
     sort_name: Optional[str] = None
     volume: Optional[int] = None
     format: Optional[str] = None
 
 
 @dataclass
-class InfoSourceMetadata:
-    source: str
-    id: int
+class CreditMetadata:
+    person: str
+    role: List[RoleMetadata]
+    id_: Optional[int] = None
 
 
 @dataclass
@@ -72,15 +76,15 @@ class GenericMetadata:
     is_empty: bool = True
     tag_origin: Optional[str] = None
 
-    info_source: Optional[InfoSourceMetadata] = None
+    info_source: Optional[GeneralResource] = None
     series: Optional[SeriesMetadata] = None
     issue: Optional[str] = None
-    stories: List[str] = field(default_factory=list)
-    publisher: Optional[str] = None
+    stories: List[GeneralResource] = field(default_factory=list)
+    publisher: Optional[GeneralResource] = None
     cover_date: Optional[date] = None
     store_date: Optional[date] = None
     issue_count: Optional[int] = None
-    genres: List[str] = field(default_factory=list)
+    genres: List[GeneralResource] = field(default_factory=list)
     language: Optional[str] = None  # 2 letter iso code
     comments: Optional[str] = None  # use same way as Summary in CIX
 
@@ -99,16 +103,17 @@ class GenericMetadata:
     page_count: Optional[int] = None
     age_rating: Optional[str] = None
 
-    story_arcs: List[str] = field(default_factory=list)
+    story_arcs: List[GeneralResource] = field(default_factory=list)
     series_group: Optional[str] = None
     scan_info: Optional[str] = None
 
-    characters: List[str] = field(default_factory=list)
-    teams: List[str] = field(default_factory=list)
-    locations: List[str] = field(default_factory=list)
+    characters: List[GeneralResource] = field(default_factory=list)
+    teams: List[GeneralResource] = field(default_factory=list)
+    locations: List[GeneralResource] = field(default_factory=list)
 
     credits: List[CreditMetadata] = field(default_factory=list)
-    tags: List[str] = field(default_factory=list)
+    reprints: List[GeneralResource] = field(default_factory=list)
+    tags: List[GeneralResource] = field(default_factory=list)
     pages: List[ImageMetadata] = field(default_factory=list)
 
     def __post_init__(self):
@@ -167,10 +172,13 @@ class GenericMetadata:
             assign("teams", new_md.teams)
         if len(new_md.locations) > 0:
             assign("locations", new_md.locations)
+        if len(new_md.reprints) > 0:
+            assign("reprints", new_md.reprints)
         assign("comments", new_md.comments)
         assign("notes", new_md.notes)
 
-        self.overlay_credits(new_md.credits)
+        if new_md.credits:
+            self.overlay_credits(new_md.credits)
 
         # For now, go the easy route, where any overlay
         # value wipes out the whole list
@@ -183,13 +191,13 @@ class GenericMetadata:
     def overlay_credits(self, new_credits: List[CreditMetadata]) -> None:
         for c in new_credits:
             # Remove credit role if person is blank
-            if c["person"] == "":
+            if c.person == "":
                 for r in reversed(self.credits):
-                    if r["role"].lower() == c["role"].lower():
+                    if r.role.lower() == c.role.lower():
                         self.credits.remove(r)
             else:
-                primary = bool("primary" in c and c["primary"])
-                self.add_credit(c["person"], c["role"], primary)
+                c.primary = bool("primary" in c and c.primary)
+                self.add_credit(c)
 
     def set_default_page_list(self, count: int) -> None:
         # generate a default page list, with the first page marked as the cover
@@ -216,21 +224,32 @@ class GenericMetadata:
 
         return coverlist
 
-    def add_credit(self, person: str, role: str, primary: bool = False) -> None:
+    def _existing_credit(self, creator: str) -> Tuple[bool, Optional[int]]:
+        return (
+            next(
+                (
+                    (True, i)
+                    for i, existing in enumerate(self.credits)
+                    if creator.lower() == existing.person.lower()
+                ),
+                (False, None),
+            )
+            if self.credits
+            else (False, None)
+        )
 
-        credit: CreditMetadata = {"person": person, "role": role, "primary": primary}
+    def _role_exists(self, new_role: RoleMetadata, old_roles: List[RoleMetadata]) -> bool:
+        return any(role.name.lower() == new_role.name.lower() for role in old_roles)
 
-        # look to see if it's not already there...
-        found = False
-        for c in self.credits:
-            if c["person"].lower() == person.lower() and c["role"].lower() == role.lower():
-                # no need to add it. just adjust the "primary" flag as needed
-                c["primary"] = primary
-                found = True
-                break
-
-        if not found:
-            self.credits.append(credit)
+    def add_credit(self, new_credit: CreditMetadata) -> None:
+        exist, idx = self._existing_credit(new_credit.person)
+        if exist:
+            existing_credit: CreditMetadata = self.credits[idx]
+            for new_role in new_credit.role:
+                if not self._role_exists(new_role, existing_credit.role):
+                    existing_credit.role.append(new_role)
+        else:
+            self.credits.append(new_credit)
 
     def __str__(self) -> str:
         vals: List[Tuple[str, str]] = []
@@ -279,6 +298,8 @@ class GenericMetadata:
             add_attr_string("teams")
         if self.locations:
             add_attr_string("locations")
+        if self.reprints:
+            add_attr_string("reprints")
         add_attr_string("comments")
         add_attr_string("notes")
 
@@ -286,9 +307,9 @@ class GenericMetadata:
 
         for c in self.credits:
             primary = ""
-            if "primary" in c and c["primary"]:
+            if "primary" in c and c.primary:
                 primary = " [P]"
-            add_string("credit", c["role"] + ": " + c["person"] + primary)
+            add_string("credit", f"{c.role}: {c.person}{primary}")
 
         # find the longest field name
         flen = 0
@@ -300,6 +321,6 @@ class GenericMetadata:
         outstr = ""
         fmt_str = "{0: <" + str(flen) + "} {1}\n"
         for i in vals:
-            outstr += fmt_str.format(i[0] + ":", i[1])
+            outstr += fmt_str.format(f"{i[0]}:", i[1])
 
         return outstr
