@@ -5,7 +5,7 @@
 import itertools
 from collections import defaultdict
 from pathlib import Path
-from typing import List, Optional
+from typing import List
 
 
 def get_recursive_filelist(pathlist: List[Path]) -> List[Path]:
@@ -38,18 +38,6 @@ def list_to_string(list_of_strings: List[str]) -> str:
     return "; ".join(map(str, list_of_strings))
 
 
-def string_to_list(string: Optional[str] = None) -> List[str]:
-    """
-    Function takes a string and converts it to a list.
-    For example: "apple; banana; cherry" is changed to ["apple", "banana", "cherry"]
-
-    :param string: A string.
-    :type string: str
-    """
-    if string is not None:
-        return [x.strip() for x in string.split(";")]
-
-
 def remove_articles(text: str) -> str:
     """
     Takes a string and removes any articles in it.
@@ -114,8 +102,5 @@ def xlate(data, is_int=False):
         )
         if i == "0":
             return "0"
-        if not i:
-            return None
-        return int(i)
-
+        return int(i) if i else None
     return str(data)
